@@ -28,37 +28,36 @@ sub on_table {
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $sb = App::DBBrowser::Table::Substatements->new( $sf->{i}, $sf->{o}, $sf->{d} );
-    my $sub_stmts = [
-        qw( print_tbl select aggregate distinct where group_by having order_by limit reset )
-    ];
-    my %cu = (
-        hidden          => 'Customize:',
-        print_tbl       => 'Print TABLE',
-        select          => '- SELECT',
-        aggregate       => '- AGGREGATE',
-        distinct        => '- DISTINCT',
-        where           => '- WHERE',
-        group_by        => '- GROUP BY',
-        having          => '- HAVING',
-        order_by        => '- ORDER BY',
-        limit           => '- LIMIT',
-        reset           => '  Reset',
+    my $hidden = 'Customize:';
+    my ( $print_table, $select, $aggregate, $distinct, $where, $group_by, $having, $order_by, $limit, $reset ) =
+       ( 'Print TABLE',
+         '- SELECT',
+         '- AGGREGATE',
+         '- DISTINCT',
+         '- WHERE',
+         '- GROUP BY',
+         '- HAVING',
+         '- ORDER BY',
+         '- LIMIT',
+         '  Reset',
     );
+    my @pre = ( $hidden, undef );
+    my @choices = ( $print_table, $select, $aggregate, $distinct, $where, $group_by, $having, $order_by, $limit, $reset );
     $sf->{i}{stmt_types} = [ 'Select' ];
     my $old_idx = 1;
 
     CUSTOMIZE: while ( 1 ) {
-        my $choices = [ $cu{hidden}, undef, @cu{@$sub_stmts} ];
+        my $menu = [ @pre, @choices ];
         $ax->print_sql( $sql );
         # Choose
         my $idx = $tc->choose(
-            $choices,
+            $menu,
             { %{$sf->{i}{lyt_v}}, prompt => '', index => 1, default => $old_idx, undef => $sf->{i}{back} }
         );
-        if ( ! defined $idx || ! defined $choices->[$idx] ) {
+        if ( ! defined $idx || ! defined $menu->[$idx] ) {
             last CUSTOMIZE;
         }
-        my $custom = $choices->[$idx];
+        my $chosen = $menu->[$idx];
         if ( $sf->{o}{G}{menu_memory} ) {
             if ( $old_idx == $idx && ! $ENV{TC_RESET_AUTO_UP} ) {
                 $old_idx = 1;
@@ -67,59 +66,59 @@ sub on_table {
             $old_idx = $idx;
         }
         my $backup_sql = $ax->backup_href( $sql );
-        if ( $custom eq $cu{'reset'} ) {
+        if ( $chosen eq $reset ) {
             $ax->reset_sql( $sql );
             $old_idx = 1;
         }
-        elsif ( $custom eq $cu{'select'} ) {
+        elsif ( $chosen eq $select ) {
             my $ok = $sb->select( $sql );
             if ( ! $ok ) {
                 $sql = $backup_sql;
             }
         }
-        elsif ( $custom eq $cu{'distinct'} ) {
+        elsif ( $chosen eq $distinct ) {
             my $ok = $sb->distinct( $sql );
             if ( ! $ok ) {
                 $sql = $backup_sql;
             }
         }
-        elsif ( $custom eq $cu{'aggregate'} ) {
+        elsif ( $chosen eq $aggregate ) {
             my $ok = $sb->aggregate( $sql );
             if ( ! $ok ) {
                 $sql = $backup_sql;
             }
         }
-        elsif ( $custom eq $cu{'where'} ) {
+        elsif ( $chosen eq $where ) {
             my $ok = $sb->where( $sql );
             if ( ! $ok ) {
                 $sql = $backup_sql;
             }
         }
-        elsif ( $custom eq $cu{'group_by'} ) {
+        elsif ( $chosen eq $group_by ) {
             my $ok = $sb->group_by( $sql );
             if ( ! $ok ) {
                 $sql = $backup_sql;
             }
         }
-        elsif ( $custom eq $cu{'having'} ) {
+        elsif ( $chosen eq $having ) {
             my $ok = $sb->having( $sql );
             if ( ! $ok ) {
                 $sql = $backup_sql;
             }
         }
-        elsif ( $custom eq $cu{'order_by'} ) {
+        elsif ( $chosen eq $order_by ) {
             my $ok = $sb->order_by( $sql );
             if ( ! $ok ) {
                 $sql = $backup_sql;
             }
         }
-        elsif ( $custom eq $cu{'limit'} ) {
+        elsif ( $chosen eq $limit ) {
             my $ok = $sb->limit_offset( $sql );
             if ( ! $ok ) {
                 $sql = $backup_sql;
             }
         }
-        elsif ( $custom eq $cu{'hidden'} ) {
+        elsif ( $chosen eq $hidden ) {
             require App::DBBrowser::Table::WriteAccess;
             my $write = App::DBBrowser::Table::WriteAccess->new( $sf->{i}, $sf->{o}, $sf->{d} );
             $write->table_write_access( $sql );
@@ -127,7 +126,7 @@ sub on_table {
             $old_idx = 1;
             $sql = $backup_sql; # so no need for table_write_access to return $sql
         }
-        elsif ( $custom eq $cu{'print_tbl'} ) {
+        elsif ( $chosen eq $print_table ) {
             local $| = 1;
             print hide_cursor(); # safety
             print clear_screen();
@@ -153,9 +152,6 @@ sub on_table {
             # return $sql explicitly since after a restore backup it refers to a different hash.
             return $all_arrayref, $sql;
         }
-        else {
-            die "'$custom': no such value in the hash \%cu";
-        }
     }
 }
 
